/*
 * Distributed as part of c3p0 v.0.9.5.1
 *
 * Copyright (C) 2015 Machinery For Change, Inc.
 *
 * Author: Steve Waldman <swaldman@mchange.com>
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of EITHER:
 *
 *     1) The GNU Lesser General Public License (LGPL), version 2.1, as 
 *        published by the Free Software Foundation
 *
 * OR
 *
 *     2) The Eclipse Public License (EPL), version 1.0
 *
 * You may choose which license to accept if you wish to redistribute
 * or modify this work. You may offer derivatives of this work
 * under the license you have chosen, or you may provide the same
 * choice of license which you have been offered here.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received copies of both LGPL v2.1 and EPL v1.0
 * along with this software; see the files LICENSE-EPL and LICENSE-LGPL.
 * If not, the text of these licenses are currently available at
 *
 * LGPL v2.1: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *  EPL v1.0: http://www.eclipse.org/org/documents/epl-v10.php 
 * 
 */

package com.mchange.v2.c3p0.test;

import javax.naming.*;
import javax.sql.*;
import com.mchange.v2.c3p0.*;

public final class JndiBindTest
{
    public static void main(String[] argv)
    {
	try
	    {
		String driverClass = null;
		String jdbc_url    = null;
		String username    = null;
		String password    = null;
		String dmds_name   = null;
		String cpds_name   = null;
		String pbds_name   = null;

		if (argv.length == 7)
		    {
			driverClass = argv[0];
			jdbc_url   = argv[1];
			username   = argv[2];
			password   = argv[3];
			dmds_name  = argv[4];
			cpds_name = argv[5];
			pbds_name = argv[6];
		    }
		else if (argv.length == 5)
		    {
			driverClass = argv[0];
			jdbc_url   = argv[1];
			username   = null;
			password   = null;
			dmds_name  = argv[2];
			cpds_name = argv[3];
			pbds_name = argv[4];
		    }
		else
		    usage();

		if (! jdbc_url.startsWith("jdbc:") )
		    usage();

		DataSource dmds = DriverManagerDataSourceFactory.create( driverClass,
									 jdbc_url, 
									 username, 
									 password );
		WrapperConnectionPoolDataSource cpds = new WrapperConnectionPoolDataSource();
		cpds.setNestedDataSource(dmds);
		DataSource pbds = PoolBackedDataSourceFactory.create( driverClass,
								      jdbc_url, 
								      username, 
								      password );

		InitialContext ctx = new InitialContext();
		ctx.rebind( dmds_name , dmds );
		System.out.println( "DriverManagerDataSource bounds as " + dmds_name );
		ctx.rebind( cpds_name , cpds );
		System.out.println( "ConnectionPoolDataSource bounds as " + cpds_name );
		ctx.rebind( pbds_name , pbds );
		System.out.println( "PoolDataSource bounds as " + pbds_name );
	    }
	catch (Exception e)
	    { e.printStackTrace(); }
    }

    private static void usage()
    {
	System.err.println("java " + JndiBindTest.class.getName() + " \\");
	System.err.println("\t<jdbc_driver_class> \\");
	System.err.println("\t<jdbc_url> [<username> <password>] \\");
	System.err.println("\t<dmds_name> <cpds_name> <pbds_name>" );
	System.exit(-1);
    }
}
